/* ----- GENERAL PURPOSE ROUTINES ----- */


#include <ctype.h>
#include "types.h"

extern RECPTR new_rec();
extern INFOPTR new_info();

RECPTR sort_merge();
STUFFPTR copy_stuff();
char *all_upper();
char *malloc();

/* ----- strallocat:  string allocate and concatinate/copy ----- */
char *strallocat(frst,sec)
	char *frst, *sec;
	{
	char *new;

	if ((new = (char *) malloc (strlen(frst)+strlen(sec)+2)) == 0)
		printf("malloc contaminated memory\n");
	sprintf(new,"%s%s\n",frst,sec);

	free(frst);
	return(new);
	}

/* ----- strcicmp: string case insensative comparison 0 or 32 terminate ----- */
int strcicmp(s1,s2)
	char *s1, *s2;
	{
	int i=0, same=0;

	do	{
		register int x=s1[i], y=s2[i];

		if ((x==0 && y==32) || (x==32 && y==0) || (x==0 && y==0)) 
			return (1);
		if ((x == 0 && (x-y) != 0))
			same = 0;
		else	{
			same = (    (x == y) ||
				    ((y - 'a' + 'A') == x) ||
				    ((x - 'a' + 'A') == y)
				) ? 1
				: 0;
			}
		i++;
		} while (same);
	return(same);
	}

/* ----- strmatch: string case insensative comparison 0 or 32 terminate ----- */
int strmatch(s1,s2,n)
	char *s1, *s2;
	int n;
	{
	int i=0, j=0, same=0;

	do	{
		register int x=s1[i++], y=s2[j++];

		if ((x == 0 && y == 32) || (x == 32 && y == 0)) 
			return (1);
		if ((x == 0 || y == 0) && (x-y) != 0)
			same = 0;
		else	{
			same = (    (x == y) ||
				    ((y - 'a' + 'A') == x) ||
				    ((x - 'a' + 'A') == y)
				) ? 1
				: 0;
			}
		} while(same && i<n && j<n);
	return(same);
	}


/* ----- all_lower:  given a string return the equivalent in lower case ----- */
all_lower(s)
	char *s;
	{
	int j=0, max;

	max = strlen(s);
	while (j<max)
		{
		s[j] = ((isalpha(s[j])&&isupper(s[j]))
			 ? tolower(s[j])
			 : s[j]);
		j++;
		}
	s[max] = '\0';

	}
/* ----- all_upper: convert a string to all upper case ----- */
char *all_upper(s)
	char *s;
	{
	char *temp;
	int i=0, j=0, max;

	if ((temp = malloc((max = strlen(s)) +1)) == 0)
		printf("malloc contaminated memory\n");

	while (j<max)
		{
		if ((s[j]==047 || s[j]=='-' || s[j]==' ' && i>0) || 
			isalpha(s[j]) || isdigit(s[j]))
			{
			temp[i++]=((isalpha(s[j])&&islower(s[j])) 
				 ? toupper(s[j]) 
				 : s[j]);
			}
		j++;
		}
	
	temp[i] = '\0';

	return(temp);
	}


/* ----- insert: add s2 to s1 from lower to upper in the given order ----- */
int insert(s1,s2,lower,upper,order)
	char *s1, *s2;
	int lower, upper, order;
	{
	int i, j, len;
	len = strlen(s2);

					/* left justified when order > 0 */
	if (order > 0)
		{
		j=lower;
		for (i=0; (j<upper && i<len); i++) 
			s1[j++] = s2[i];
		j--;
		}
					/* right justified when order < 0 */
	else if (order < 0)
		{
		j=upper;
		for(i=(len-1); (j>=lower && i>=0); i--) 
			s1[j--] = s2[i];
		j++;
		}
					/* centered when order == 0 */
	else	{
		j = (int)((upper - lower - len)/2) + lower;
		for (i=0; (i<len && j<upper && j>=0); i++)
			s1[j++] = s2[i];
		}

	return (j);
	}

/* ----- end_it: given a buffer, end the string with a null ----- */
end_it(buf,max)
	char *buf;
	int max;
	{
	int i=0, done=0;

	do	{
		register int j;

		while (i<(max-1) && !isspace(buf[i])) i++;
		done = 1;
					/* look ahead for intermediate ws */
		j=i;
		while (j<(i+5) && j<(max-1) && done)
			{
			if (!isspace(buf[j]))
				{
				done = 0;
				i=j;
				}
			j++;
			}
		} while (!done);
	buf[i] = '\0';
	}


/* ----- make_name: convert all caps to name style case ----- */
char *make_name(s)
	char *s;
	{
	char *temp;
	int i=0, j=0, done=0, roman=0;

	if (strlen(s) == 0)
		return((char *)0);

	if ((temp = (char *) malloc (strlen(s)+1)) == 0)
		printf("malloc failed\n");

	while (!(done = (s[i] == '\0')) && isspace(s[i])) i++;

	do	{
		roman = 0;
						/* save in between stuff */
		while (!(done = (s[i]=='\0')) && !(isalpha(s[i])))
			temp[j++] = s[i++];
						/* capitalize first letter */
		if (!done && isalpha(s[i]))
			{
			temp[j++] = (islower(s[i]) ? toupper(s[i]) : s[i]);
			roman = isroman(s,i);
			i++;
			}
						/* lower case rest of name */
		while ( !(done = (s[i] == '\0')) && isalpha(s[i]))
			{
						/* check for roman numerals */
			if (roman && (roman = isroman(s,i)))
				{
				temp[j++] = (islower(s[i]) ? toupper(s[i]) : s[i]);
				}
			else	{

				temp[j++]=(isupper(s[i]) ? tolower(s[i]) : s[i]);
				}
			i++;
			}
						/* check for a buffer */
		if (!done)
			if (isspace(s[i+1]) && isspace(s[i+2])) done =1;
		} while (!done);

	temp[j] = '\0';

	return(temp);
	}


/* ----- same_dept: compare two department names return 1 if similar ----- */
int same_dept(d1,d2)
	char *d1, *d2;
	{
	int i1=0, i2=0, l1, l2, same=1;
	register char a,b;

	l1 = strlen(d1); l2 = strlen(d2);

	if (l1 == 0 && l2 == 0)
		return(1);

	while(same && i1<l1 && i2<l2)
		{
		a = d1[i1]; b = d2[i2];
		if (isalpha(a) && isalpha(b))
			{
			if (a != b) same =0;
			i1++; i2++;
			}
		else	{
			while (i1<l1 && !isalpha(d1[i1])) i1++;
			while (i2<l2 && !isalpha(d2[i2])) i2++;
			}
		}
	return(same);
	}


/* ----- sort_merge:  sort and merge the two records ----- */
RECPTR sort_merge(rec1, rec2)
	RECPTR rec1, rec2;
	{
	RECPTR sorted, unsorted;
	STUFFPTR place, node;

	sorted = new_rec();

	if (rec1 != (RECPTR)0 && rec1->num > 0)
		{
		unsorted = rec1;
		if (rec2 != (RECPTR)0 && rec2->num > 0)
			{
			unsorted->tail->_next = rec2->head;
			unsorted->tail = rec2->tail;
			}
		}
	else if (rec2 != (RECPTR)0 && rec2->num > 0)
		unsorted = rec2;
	else	return(sorted);
	
	place = unsorted->head;

	while(place != (STUFFPTR)0)
		{
		node = copy_stuff(place);
		add_list(sorted, node);
		place = place->_next;
		}

	return(sorted);
	}

/* ----- add_list:  add a node to a sorted list ----- */
add_list(list, node)
	RECPTR list;
	STUFFPTR node;
	{
	if (list->num == 0)
		{
		list->head = node;
		list->tail = node;
		(list->num)++;
		}
	else	{
		STUFFPTR cur, last;
		int placed=0;
		int c;

		last = cur = list->head;

		while (last != (STUFFPTR)0 && !placed)
			{
			if ((c=strcmp(cur->info, node->info)) == 0)
				placed = 1;
			else if (c > 0)
				{
				placed = 1;
				(list->num)++;
				if (cur == last)
					{
					node->_next = list->head;
					list->head = node;
					}
				else	{
					last->_next = node;
					node->_next = cur;
					if (cur == (STUFFPTR)0)
						list->tail = node;
					}
				}
			else	{
				last = cur;
				cur = last->_next;
				}
			}
		if (!placed)
			printf("warning...node not placed in add_list()\n");

		}
	}

/* ----- copy_stuff: make a new node with a copy of a stuff node ----- */
STUFFPTR copy_stuff(s)
	STUFFPTR s;
	{
	STUFFPTR val;

	if ((val = (STUFFPTR)malloc(sizeof(STUFFNODE))) == 0)
		printf("malloc contaminated memory\n");
	if (s->info != (INFOPTR)0)
		{
		val->info = new_info(strlen(s->info)+1);
		strcpy(val->info, s->info);
		}
	else	val->info = (INFOPTR)0;

	if (s->long_info != (INFOPTR)0)
		{
		val->long_info = new_info(strlen(s->long_info)+1);
		strcpy(val->long_info, s->long_info);
		}
	else	val->long_info = (INFOPTR)0;

	val->_next = (STUFFPTR)0;

	return(val);
	}

int isroman(s,i)
	char *s;
	int i;
	{
	int val=0, j=i;
	char c;

	c = s[j++];
	if ((c == 'i' || c == 'I' || c == 'v' || c == 'V' || c == 'x' || c == 'X') &&
	    (!isalpha(s[j]) || isroman(s,j)))
		val = 1;
	return(val);
	}
