/* ----- SEARCH ROUTINES FOR DATABASE ----- */

#include <stdio.h>
#include "/usr/unify/include/fdesc.h"
#include "/usr/unify/include/dbtypes.h"
#include "/usr/unify/include/btincl.h"
#include "file.h"
#include "types.h"

extern int strcicmp(), strmatch(), valid();
extern add_rec(), end_it();
extern pr_rec();

RECPTR new_rec(), search(), merge();
char *malloc();

int DIRTY=0;

/* ----- find: given a set of btptrs and searches, return all found ----- */
RECPTR find (btptrs, wild, packet)
	char **btptrs;
	int *wild;
	PACKET packet;
	{
	RECPTR value, temp;
	int i=0, rnum, fnum;

	value = (RECPTR)0;
				/* add the long info once for first entry */
				/* to keep track, set a dirty flag when done */
	DIRTY=0;

	while ((rnum=packet[i][0]) > 0 && (fnum=packet[i][1]) > 0)
		{
		temp = search(btptrs,wild,rnum,fnum);

		if (value == (RECPTR)0 || value->head == (STUFFPTR)0)
			value = temp;
		else if (temp->num > 0)
			{
			value = merge(value, temp);
			}
		i++;
		}
	return (value);
	}


/* ----- merge:  merge rec2 with rec1 and return the result ----- */
RECPTR merge(rec1, rec2)
	RECPTR rec1, rec2;
	{
	STUFFPTR place, temp, cur, last;

/*
printf("begin merging lists\n");
pr_rec(rec1);
pr_rec(rec2);
*/
				/* quick check for a nil list */
	if (rec2 == (RECPTR)0 && rec1 != (RECPTR)0)
		return (rec1);
	else if (rec2 != (RECPTR)0 && rec1 == (RECPTR)0)
		return (rec2);

	rec1->num = rec1->num + rec2->num;

	place = rec2->head; cur = last = rec1->head;

	while(place != (STUFFPTR)0 && place->info != (INFOPTR)0)
		{
		int placed=0;
/*
printf("placing: %s\n",place->info);
*/
					/* add one rec2 to rec1 list */
		while (!placed && cur != (STUFFPTR)0)
			{
			if (strcmp(cur->info, place->info) > 0)
				{
				temp = place->_next;
				if (cur == last)
					{
/*
printf("before head, %s\n",rec1->head->info);
*/
					place->_next = rec1->head;
					rec1->head = place;
					last = rec1->head;
					}
				else	{
/*
printf("in the middle before, %s\n",cur->info);
*/
					place->_next = cur;
					last->_next = place;
					last = place;
					}
				place = temp;
				placed =1;
				}
			else	{
				last = cur;
				}
			cur = last->_next;
			}
		if (!placed)
			{
/*
printf("at the end after, %s\n",rec1->tail->info);
*/
			temp = place->_next;
			place->_next = (STUFFPTR)0;
			rec1->tail->_next = place;
			rec1->tail = place;
			place = temp;
			}
		}
/*
printf("end merging lists\n");
*/
	return(rec1);
	}

/* ----- search:  search the btree for all occurances of a given field ----- */
RECPTR search (btptrs, wild, rnum, fnum)
	char **btptrs;
	int *wild;
	int rnum, fnum;
	{
	RECPTR value;
	int same=0;
	int rc, btree;

	value = new_rec();

	if ((btree = in_btree(btptrs,fnum)) < 0)
		{
		return((RECPTR)0);
		}

	do	{
		if (rnum==Mailid && fnum==mlstname && btptrs[1]!=(char *)0)
			{
			if (same = matchflds(btptrs[0],wild[0],rnum,fnum))
				if (matchflds(btptrs[1],wild[1],rnum,mfstname))
					add_rec(value, rnum);
			}
		else if (same = matchflds(btptrs[0],wild[0],rnum,fnum))
			{
			add_rec(value,rnum);
			}

		} while (same && btnext(btree) == 0);

/*
printf("closing btree %d after one search\n",btree);
*/
	if (rc = closbt(btree))
		{
		fprintf(stderr,"Closbt failed rc = %d\n",rc);
		exit(1);
		}
	return(value);
	}

/* ----- in_btree: check for a string in the given btree ----- */
int in_btree (btptrs, fnum)
	char **btptrs;
	int fnum;
	{
	int rc, status, btree;
	BTIDS *bts, *fldidxd();

/*
printf("in_btree:  fnum = %d, value = [%s]\n",fnum,btptrs[0]);
*/
	if((bts = fldidxd(fnum, &status)) == BTIDNUL)
		{
		if (status == IFNBT_R)
			{
			fprintf(stderr,"Field %d is not indexed\n",fnum);
			exit(1);
			}
		else	{
			fprintf(stderr,"Internal error %d returned by fldidxd.\n",status);
			exit(1);
			}
		}
	if(bts[0].btfrank !=0)
		{
		fprintf(stderr,"Field must be the first element of the index.\n");
		exit(1);
		}
	btree = bts[0].btidnum;
/*
printf("IN_BTREE:  trying to open btree %d, string [%s], fnum %d\n",btree,btptrs[0],fnum);
*/

	if (rc = opnbts(btree))
		{
		fprintf(stderr,"Open btree failed rc = %d\n",rc);
		exit(1);
		}

	if(btsrch(btree,btptrs) < 0)
		return(-1);

/* CHECK */
/*
gfield(mlstname,buf);
end_it(buf,64);
printf("after searching the btree for %s, ended on %s\n", btptrs[0], buf);
*/

	return(btree);
	}

/* ----- matchflds: return information for a matching key ----- */
int matchflds(string,wild,rnum,fnum)
	char *string;
	int wild;
	int rnum, fnum;
	{
	int val=0;
	struct fdesc f;
	/*
	int unify_sucks;		/* fldesc() munges my var decls *
	*/

	if (fldesc(fnum,&f) == 0)
		{
		return(val);
		}
	if (f.f_rec != rnum)
		{
		return(val);
		}
	if (f.f_typ == STRNG) 
		{
		char buffer[32];

		gfield(fnum,buffer);
		end_it(buffer,32);

		if (strcicmp(string,buffer))
			{
			val =1;
			}
		else if (wild && strmatch(string, buffer, strlen(string)))
			{
			val =1;
			}
		else 	val = 0;
		}
	else	{
		fprintf(stderr,"undefined Unify type to matchflds\n");
		}
	return(val);
	}

/* ----- new_rec:  new record to hold the numbers ----- */
RECPTR new_rec()
	{
	RECPTR node;

	if ((node = (RECPTR)malloc(sizeof(RECNODE))) == 0)
		printf("malloc contaminated memory\n");

	node->num=0;
	node->head =(STUFFPTR)0;
	node->tail =(STUFFPTR)0;

	return(node);
	}

